@@ -3,10 +3,15 @@
  * sorts a recipient list by reversed domain and splits it up into
  * several envelopes so sendmail can deliver it in parallel.
  *
- * written March 1994
- * by Keith Moore <moore@cs.utk.edu>
+ * Copyright 1994,1995,1996,1997,1998 by Keith Moore <moore@cs.utk.edu>.
+ * All rights reserved.
+ * 
+ * License to copy and use this program is granted according to the terms
+ * of the current version of the GNU General Public License.  However,
+ * there is one exception: this program may not be used to send unsolicited  
+ * commercial messages.  
  *
- * You can use this if you want to, but there's no warranty.
+ * There's no warranty on this program.
  * So don't blame me if something goes wrong.
  */
 
@@ -144,6 +149,17 @@ char *sendmail_flags = "";	/* call sendmail with these flags */
 int subscribe_flag = 0;		/* check for "[un]subscribe" in body */
 int verbose_flag = 0;		/* be verbose */
 
+/*
+ * List-* headers.  See RFC 2369 before setting these.
+ */
+char *list_archive_hdr = NULL;
+char *list_help_hdr = NULL;
+char *list_owner_hdr = NULL;
+char *list_post_hdr = NULL;
+char *list_subscribe_hdr = NULL;
+char *list_unsubscribe_hdr = NULL;
+int del_list_hdrs = 0;
+
 /*
  * structure used to hold an address while sorting
  */
@@ -513,8 +529,16 @@ FILE *fp;
 {
     char linebuf[1024];
 
-    while (fgets (linebuf, sizeof (linebuf), fp) != NULL)
+    while (fgets (linebuf, sizeof (linebuf), fp) != NULL) {
+	char *ptr = linebuf;
+	
+	while (*ptr == ' ' || *ptr == '\t')
+	    ++ptr;
+	if (*ptr == '#')
+	    continue;
+	
 	process_addr (linebuf);
+    }
 }
 
 /*
@@ -738,9 +762,9 @@ char *buf, *str;
     for (ptr = buf; *ptr; ++ptr) {
 	if (*ptr == c1 || *ptr == c2) {
 	    if ((ptr == buf || ptr[-1] == ' ' || ptr[-1] == '\t') &&
-		strncasecmp (buf, str, length) == 0 &&
+		strncasecmp (ptr, str, length) == 0 &&
 		(ptr[length] == ' ' || ptr[length] == '\t' ||
-		 ptr[length] == '\n'))
+		 ptr[length] == '\n' || ptr[length] == '\0'))
 		return 1;
 	}
     }
@@ -911,6 +935,7 @@ time_t *t;
 #define MESSAGE_OKAY		0
 #define HAS_EMBEDDED_COMMAND	1
 #define LOOKS_LIKE_BOUNCE	2
+#define WRITE_ERROR		3
 
 static int
 copy_message (out, in)
@@ -982,6 +1007,18 @@ FILE *out, *in;
 	    }
 	    else if (match_hdr ("errors-to", linebuf))
 		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-archive", linebuf))
+		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-help", linebuf))
+		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-owner", linebuf))
+		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-post", linebuf))
+		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-subscribe", linebuf))
+		goto delete_header;
+	    else if (del_list_hdrs && match_hdr ("list-unsubscribe", linebuf))
+		goto delete_header;
 	    else if (match_hdr ("message-id", linebuf))
 		has_message_id_hdr = 1;
 	    else if (match_hdr ("precedence", linebuf))
@@ -1098,6 +1135,20 @@ FILE *out, *in;
 		 getpid(), tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
 		 tm->tm_hour, tm->tm_min, tm->tm_sec, local_domain);
     }
+
+    if (list_archive_hdr)
+	fprintf (out, "List-Archive: %s\n", list_archive_hdr);
+    if (list_help_hdr)
+	fprintf (out, "List-Help: %s\n", list_help_hdr);
+    if (list_owner_hdr)
+	fprintf (out, "List-Owner: %s\n", list_owner_hdr);
+    if (list_post_hdr)
+	fprintf (out, "List-Post: %s\n", list_post_hdr);
+    if (list_subscribe_hdr)
+	fprintf (out, "List-Subscribe: %s\n", list_subscribe_hdr);
+    if (list_unsubscribe_hdr)
+	fprintf (out, "List-Unsubscribe: %s\n", list_unsubscribe_hdr);
+
     fprintf (out, "\n");
 
     /*
@@ -1123,11 +1174,14 @@ FILE *out, *in;
      * a special code so that our caller will know to forward us to
      * the human.
      */
+    fflush (out);
     if (has_valid_approved_hdr == 0 && looks_like_bounce)
 	return LOOKS_LIKE_BOUNCE;
     else if (has_valid_approved_hdr == 0 && saw_command > 0 && 
 	     subscribe_flag && lines <= 7)
 	return HAS_EMBEDDED_COMMAND;
+    else if (ferror (out))
+	return WRITE_ERROR;
     else
 	return MESSAGE_OKAY;
 }
@@ -1226,6 +1280,8 @@ char *argv[];
 	}
 	else if (strcmp (argv[1], "-debug") == 0)
 	    ++debug_flag;
+	else if (strcmp (argv[1], "-delete-list-hdrs") == 0)
+	    ++del_list_hdrs;
 	else if (strcmp (argv[1], "-domain") == 0 && argc > 2) {
 	    if (strchr (argv[2], '.') == NULL) {
 		fprintf (stderr,
@@ -1237,6 +1293,42 @@ char *argv[];
 	    --argc;
 	    ++argv;
 	}
+	else if (strcmp (argv[1], "-list-archive") == 0 && argc > 2) {
+	    list_archive_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-list-help") == 0 && argc > 2) {
+	    list_help_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-list-owner") == 0 && argc > 2) {
+	    list_owner_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-list-post") == 0 && argc > 2) {
+	    list_post_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-list-subscribe") == 0 && argc > 2) {
+	    list_subscribe_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-list-unsubscribe") == 0 && argc > 2) {
+	    list_unsubscribe_hdr = argv[2];
+	    del_list_hdrs = 1;
+	    --argc;
+	    ++argv;
+	}
 	else if (strcmp (argv[1], "-maxdomains") == 0 && argc > 2) {
 	    max_domains = atoi (argv[2]);
 	    if (max_domains <= 0 || !digits_only (argv[2])) {
@@ -1298,21 +1390,31 @@ char *argv[];
 	fprintf (stderr,
 		 "usage: bulk_mailer [options] envelope_from recipient_list_file\n");
 	fprintf (stderr, "(message appears on stdin)\n");
-	fprintf (stderr, "options are (NOTE: NR = NOT RECOMMENDED):\n");
+	fprintf (stderr, "options are:\n");
 	fprintf (stderr, "\
     -comment text          add a header of the form \"Comment: text\"\n\
     -debug                 display SMTP output on stderr; don't send msg\n\
+    -delete-list-hdrs      delete any List-* headers in input\n\
     -domain xx.yy.zz       set domain name to 'xx.yy.zz' (overrides default)\n\
+    -list-archive xxx      add a 'List-Archive: xxx' field to output (*)\n\
+    -list-help xxx         add a 'List-Help: xxx' field to output (*)\n\
+    -list-owner xxx        add a 'List-Owner: xxx' field to output (*)\n\
+    -list-post xxx         add a 'List-Post: xxx' field to output (*)\n\
+    -list-unsubscribe xxx  add a 'List-Unsubscribe: xxx' field to output (*)\n\
     -maxdomains ###        set max # of distinct domains per envelope\n\
     -maxsize ####          set maximum message size\n\
     -owner user@domain     set name of list owner\n\
-    -precedence keyword    add a \"Precedence: keyword\" header field (NR)\n\
+    -precedence keyword    add a 'Precedence: keyword' header field (NR)\n\
     -reply-to user@domain  add a reply-to header if there's not one there\n\
     +reply-to user@domain  force reply-to header even if there already (NR)\n\
     -s                     check for 'unsubscribe' & similar commands\n\
                            (if found: don't deliver, forward to list owner)\n\
     -sendmail flags        pass 'flags' to sendmail\n\
-    -v                     turn on verbose mode\n");
+    -v                     turn on verbose mode\n\
+    \n\
+    NOTES:\n\
+    (NR) = NOT RECOMMENDED\n\
+     (*) = read RFC 2369 before setting these options\n");
 	exit (EX_USAGE);
     }
     envelope_from = argv[1];
@@ -1350,6 +1452,13 @@ char *argv[];
 	unlink (tempname);
 	exit (EX_OK);
 
+    case WRITE_ERROR:
+	fprintf (stderr, "bulk_mailer: error writing temp file:\n");
+	fprintf (stderr, "%s\n", strerror (errno));
+	fclose (tmp);
+	unlink (tempname);
+	exit (EX_TEMPFAIL);
+
     case MESSAGE_OKAY:
 	if (size_of_file (fileno (tmp)) > max_size) {
 	    fprintf (stderr,
